@@ -121,7 +121,8 @@ def _external_create(project_name: str, key: str = ''):
 
         try:
             from QuickStart_Rhy.API.alapi import ip_info
-            with QproDefaultConsole.status(('Check IP to switch mirrors' if user_lang != 'zh' else '检查IP以选择合适的镜像').format(key, project_name)):
+            with QproDefaultConsole.status(
+                    ('Check IP to switch mirrors' if user_lang != 'zh' else '检查IP以选择合适的镜像').format(key, project_name)):
                 is_CN = ip_info('')['ad_info']['nation'].startswith('中国')
         except:
             is_CN = False
@@ -129,7 +130,9 @@ def _external_create(project_name: str, key: str = ''):
         templateProjectUrls = _search_supported_languages(is_CN)
         if not templateProjectUrls:
             exit(0)
-        with QproDefaultConsole.status(('Cloning Qpro {} Template to {}' if user_lang != 'zh' else '正在克隆Qpro {} 模板为 {}').format(key, project_name)):
+        with QproDefaultConsole.status(
+                ('Cloning Qpro {} Template to {}' if user_lang != 'zh' else '正在克隆Qpro {} 模板为 {}').format(key,
+                                                                                                         project_name)):
             Repo.clone_from(templateProjectUrls[0], project_name)
     else:
         templateProjectUrls = _ask({
@@ -137,7 +140,8 @@ def _external_create(project_name: str, key: str = ''):
             'message': 'GIT ' + ('URL' if user_lang != 'zh' else '链接') + ':',
             'name': 'url'
         })
-        with QproDefaultConsole.status(('Cloning External Template to {}' if user_lang != 'zh' else '正在克隆Qpro 外部模板为 {}').format(project_name)):
+        with QproDefaultConsole.status(
+                ('Cloning External Template to {}' if user_lang != 'zh' else '正在克隆Qpro 外部模板为 {}').format(project_name)):
             Repo.clone_from(templateProjectUrls, project_name)
     os.chdir(project_name)
     try:
@@ -152,7 +156,8 @@ def create():
     try:
         project_name = sys.argv[2]
     except IndexError:
-        return QproDefaultConsole.print(QproWarnString, 'usage: Qpro create <project>' if user_lang != 'zh' else '使用: Qpro create <项目>')
+        return QproDefaultConsole.print(QproWarnString,
+                                        'usage: Qpro create <project>' if user_lang != 'zh' else '使用: Qpro create <项目>')
     else:
         if os.path.exists(project_name) and os.path.isdir(project_name):
             return QproDefaultConsole.print(QproErrorString, '"%s" is exist!' % (os.path.abspath(project_name)))
@@ -246,7 +251,8 @@ def get():
         )
     else:
         if not sub_path:
-            return QproDefaultConsole.print(QproErrorString, f"{path} is not in this Qpro project!" if user_lang != 'zh' else f'{path} 不在当前 Qpro 项目中!')
+            return QproDefaultConsole.print(QproErrorString,
+                                            f"{path} is not in this Qpro project!" if user_lang != 'zh' else f'{path} 不在当前 Qpro 项目中!')
         server, target, port = get_server_target()
         user, ip = server.split('@') if '@' in server else [None, server]
         SshProtocol.get_file_or_folder(user, ip, target, port, sub_path, path)
@@ -315,12 +321,12 @@ def adjust():
                                             '        addr1: <username>@<ipv4 | ipv6 | domain>:</path/to/project/>\n'
                                             '        addr2: <configured name>:</path/to/project/>\n'
                                             '        port : <ssh port>, default 22') \
-                        if user_lang != 'zh' else \
-                        QproDefaultConsole.print(
-                            QproWarnString, '合法的远程映射:\n'
-                                            '        地址1: <用户名>@<ipv4 | ipv6 | 域名>:</项目/路径/>\n'
-                                            '        地址2: <SSH配置表中项目>:</项目/路径/>\n'
-                                            '        端口 : <SSH 端口>, 默认 22')
+                            if user_lang != 'zh' else \
+                            QproDefaultConsole.print(
+                                QproWarnString, '合法的远程映射:\n'
+                                                '        地址1: <用户名>@<ipv4 | ipv6 | 域名>:</项目/路径/>\n'
+                                                '        地址2: <SSH配置表中项目>:</项目/路径/>\n'
+                                                '        端口 : <SSH 端口>, 默认 22')
             else:
                 config[dt] = all_dt[dt].get()
         if not config['template_root'].endswith(dir_char):
@@ -330,7 +336,8 @@ def adjust():
         win.destroy()
         __format_json(config, project_configure_path)
 
-    tk.Button(win, text='确认', command=deal_config, width=10).grid(row=7 if 'enable_complete' in config else 6, column=0, columnspan=3)
+    tk.Button(win, text='确认', command=deal_config, width=10).grid(row=7 if 'enable_complete' in config else 6, column=0,
+                                                                  columnspan=3)
     tk.mainloop()
 
 
@@ -360,10 +367,10 @@ def pro_init():
         source_file = ('main' + lang[-1]) if lang[0] != 'javac' else work_project + lang[-1]
         while not os.path.exists(source_file):
             source_file = Prompt.ask((
-                            'Not found "%s", set compile_filename'
-                            if user_lang != 'zh' else
-                            '没有找到 "%s", 请设置源文件'
-                            ) % source_file).strip()
+                                         'Not found "%s", set compile_filename'
+                                         if user_lang != 'zh' else
+                                         '没有找到 "%s", 请设置源文件'
+                                     ) % source_file).strip()
     server_target = Prompt.ask(
         'input user@ip:dir_path if you need scp' if user_lang != 'zh' else '输入 用户@IP:路径 如果你打算使用SSH'
     ).strip().replace(dir_char, '/')
@@ -489,6 +496,200 @@ def enable_complete():
     __format_json(config, project_configure_path)
 
 
+def __get_Qpro_fig_Dir():
+    QproGlobalDir = os.environ.get('QproGlobalDir', None)
+    if not QproGlobalDir:
+        QproDefaultConsole.print(QproErrorString,
+                                 'QproGlobalDir is not set!' if user_lang != 'zh' else 'QproGlobalDir 未设置!')
+        return None
+    QproGlobalDir = QproGlobalDir.replace('~', user_root)
+    if not os.path.exists(QproGlobalDir):
+        QproDefaultConsole.print(QproErrorString,
+                                 'QproGlobalDir is not exists!' if user_lang != 'zh' else 'QproGlobalDir 不存在!')
+        return None
+    if not os.path.exists(os.path.join(QproGlobalDir, 'bin')):
+        os.mkdir(os.path.join(QproGlobalDir, 'bin'))
+    if not os.path.exists(os.path.join(QproGlobalDir, 'fig')):
+        os.mkdir(os.path.join(QproGlobalDir, 'fig'))
+    return QproGlobalDir
+
+
+def register_global_command():
+    if is_win:
+        return QproDefaultConsole.print(QproWarnString, 'Not Support Windows!' if user_lang != 'zh' else '不支持 Windows!')
+    QproGlobalDir = __get_Qpro_fig_Dir()
+    if not QproGlobalDir:
+        return
+    import json
+
+    disable_global_command = '--disable_global_command' in sys.argv and int(sys.argv[sys.argv.index('--disable_global_command') + 1])
+    disable_global_command = True if disable_global_command else False
+    print(disable_global_command)
+    project_name = os.getcwd().split(dir_char)[-1]
+    fig_dir = os.path.join(QproGlobalDir, 'fig')
+    if os.path.exists(os.path.join(fig_dir, f'{project_name}.json')):
+        QproDefaultConsole.print(QproWarnString,
+                                 'This project has been registered!' if user_lang != 'zh' else '该项目已注册!')
+        if _ask({
+            'type': 'confirm',
+            'name': 'confirm',
+            'message': 'Do you want to override it?' if user_lang != 'zh' else '是否覆盖?',
+            'default': False
+        }):
+            os.remove(os.path.join(fig_dir, f'{project_name}.json'))
+        else:
+            return
+    import subprocess
+
+    with open(os.path.join(fig_dir, f'{project_name}.json'), 'w') as f:
+        project_subcommands = json.loads(subprocess.check_output(['qrun', '--qrun-fig-complete']).decode('utf-8'))
+        if not project_subcommands:
+            return QproDefaultConsole.print(QproErrorString, 'Not a Commander APP' if user_lang != 'zh' else '不是Commander应用')
+        for item in project_subcommands:
+            if 'args' in item:
+                if 'options' not in item:
+                    item['options'] = []
+                for arg in item['args']:
+                    if arg['name'].startswith('-') and not arg['name'].startswith('--'):
+                        if 'file' in arg['name'] or 'path' in arg['name']:
+                            arg['args']['template'] = ['filepaths', 'folders']
+                        item['options'].append(arg)
+                        item['args'].remove(arg)
+                        continue
+                    if 'file' in arg['name'] or 'path' in arg['name']:
+                        arg['template'] = ['filepaths', 'folders']
+
+        json.dump({
+            'fig': {
+                'name': project_name,
+                'description': project_name,
+                'subcommands': project_subcommands
+            },
+            'path': os.getcwd()
+        }, f, ensure_ascii=False, indent=1)
+    if disable_global_command:
+        return
+    with open(os.path.join(QproGlobalDir, 'bin', f'{project_name}'), 'w') as f:
+        with open(get_config()['compile_filename'], 'r') as ff:
+            ct = ff.read()
+        if not ct.startswith('#!/usr/bin/env python3'):
+            ct = '#!/usr/bin/env python3\n' + ct
+        f.write(ct)
+    os.chmod(os.path.join(QproGlobalDir, 'bin', f'{project_name}'), 0o755)
+
+
+def gen_fig_script():
+    if is_win:
+        return QproDefaultConsole.print(QproWarnString, 'Not Support Windows!' if user_lang != 'zh' else '不支持 Windows!')
+    QproGlobalDir = __get_Qpro_fig_Dir()
+    if not QproGlobalDir:
+        return
+    import json
+    from .QproFigTable import default_custom_command_template
+    fig_Qpro_template = requirePackage('.QproFigTable', f'default_Qpro_{user_lang if user_lang == "zh" else "en"}_content_template')
+    fig_qrun_template = requirePackage('.QproFigTable', f'default_qrun_{user_lang if user_lang == "zh" else "en"}_content_template')
+    fig_detector_template = requirePackage('.QproFigTable', f'default_detector_{user_lang if user_lang == "zh" else "en"}_content_template')
+    fig_tmpm_template = requirePackage('.QproFigTable', f'default_tmpm_{user_lang if user_lang == "zh" else "en"}_content_template')
+
+    with open('src/Qpro.ts', 'w') as f:
+        subcommands = os.listdir(os.path.join(QproGlobalDir, 'fig'))
+        res = []
+        for cmd in subcommands:
+            with open(os.path.join(QproGlobalDir, 'fig', cmd), 'r') as cmdf:
+                proj_name = cmd.split(".")[0]
+                fig = json.load(cmdf)['fig']
+                res.append(fig)
+                if not os.path.exists(os.path.join(QproGlobalDir, 'bin', f'{proj_name}')):
+                    continue
+                with open(f'src/{proj_name}.ts', 'w') as custom_f:
+                    custom_f.write(default_custom_command_template.replace('__CUSTOM_COMMAND_SPEC__', json.dumps(fig, ensure_ascii=False, indent=1)))
+        f.write(fig_Qpro_template.replace('--Qpro-bin-subcommands-gen--', json.dumps(res, ensure_ascii=False, indent=1)))
+    with open('src/qrun.ts', 'w') as f:
+        f.write(fig_qrun_template)
+    with open('src/detector.ts', 'w') as f:
+        f.write(fig_detector_template)
+    with open('src/tmpm.ts', 'w') as f:
+        f.write(fig_tmpm_template)
+    os.system('npm run copy-to-global')
+
+
+def bin_exec():
+    if is_win:
+        return QproDefaultConsole.print(QproWarnString, 'Not Support Windows!' if user_lang != 'zh' else '不支持 Windows!')
+    import json
+    proj_name = sys.argv[2]
+    proj_args = sys.argv[3:]
+    QproGlobalDir = __get_Qpro_fig_Dir()
+    if not QproGlobalDir:
+        return
+    if f'{proj_name}.json' not in os.listdir(os.path.join(QproGlobalDir, 'fig')):
+        return QproDefaultConsole.print(QproErrorString, f'{proj_name} is not registered' if user_lang == 'zh' else f'{proj_name} 未注册')
+    with open(os.path.join(QproGlobalDir, 'fig', f'{proj_name}.json'), 'r') as f:
+        path = json.load(f)['path']
+    os.chdir(path)
+    os.system(f'qrun ' + ' '.join(proj_args))
+
+
+def gen_zsh_comp():
+    if is_win:
+        return QproDefaultConsole.print(QproWarnString, 'Not Support Windows!' if user_lang != 'zh' else '不支持 Windows!')
+    QproGlobalDir = __get_Qpro_fig_Dir()
+    if not QproGlobalDir:
+        return
+
+    import json
+    from .QproZshComp import zsh_comp_template
+
+    subcommands = os.listdir(os.path.join(QproGlobalDir, 'fig'))
+    file_comp1 = """
+    else
+        _arguments -S -s '*:filename:_files'
+        return
+    fi
+    """
+    file_comp2 = """
+    _arguments -S -s '*:filename:_files'
+    return
+    """
+    file_comp3 = """
+    if [[ ${prev} == __sub_cmd__ ]]; then
+        _arguments -S -s '*:filename:_files'
+        return
+    """
+    for cmd in subcommands:
+        proj_name = cmd.split(".")[0]
+        if not os.path.exists(os.path.join(QproGlobalDir, 'bin', f'{proj_name}')):
+            continue
+        with open(os.path.join(QproGlobalDir, 'fig', cmd), 'r') as cmdf:
+            fig = json.load(cmdf)['fig']
+        cur_sub_cmds = []
+        sub_cmd_args = []
+        for sub_cmd in fig['subcommands']:
+            cur_sub_cmds.append(f"{sub_cmd['name']}:'{sub_cmd['description']}'")
+            if 'args' in sub_cmd and sub_cmd['args']:
+                cur_args = """if [[ ${prev} == __sub_cmd__ ]]; then
+        opt_args=(
+            __sub_cmd_opts__
+        )"""
+                sub_cmd_opts = []
+                for arg in sub_cmd['args']:
+                    if arg['name'].startswith('-'):
+                        sub_cmd_opts.append(f"{arg['name']}:'{arg['description']}'")
+                for opt in sub_cmd['options']:
+                    if opt['name'].startswith('-'):
+                        sub_cmd_opts.append(f"{opt['name']}:'{opt['description']}'")
+                cur_args = cur_args.replace('__sub_cmd__', sub_cmd['name'])
+                cur_args = cur_args.replace('__sub_cmd_opts__', '\n            '.join(sub_cmd_opts))
+                sub_cmd_args.append(cur_args)
+
+        with open('_' + fig['name'], 'w') as f:
+            template = zsh_comp_template
+            template = template.replace('__proj_name__', fig['name'])
+            template = template.replace('__sub_commands__', '\n        '.join(cur_sub_cmds))
+            template = template.replace('__sub_commands_args__', '\n    el'.join(sub_cmd_args) + file_comp1 if sub_cmd_args else file_comp2)
+            f.write(template)
+
+
 func = {
     'create': create,
     'scp': scp,
@@ -500,7 +701,11 @@ func = {
     'del': delete,
     'ls': tele_ls,
     'csv': template_format,
-    'enable-complete': enable_complete
+    'bin': bin_exec,
+    'enable-complete': enable_complete,
+    'register-global': register_global_command,
+    'gen-fig-script': gen_fig_script,
+    'gen-zsh-comp': gen_zsh_comp,
 }
 
 
@@ -508,46 +713,54 @@ def main():
     if len(sys.argv) < 2 or '-h' == sys.argv[1]:
         menu_output({'title': 'Qpro usage\n' if user_lang != 'zh' else 'Qpro 菜单\n',
                      'lines': [
-                        ('init', 'let current dir be a Qpro project!' if user_lang != 'zh' else '使当前目录成为Qpro项目'),
-                        ('-h', 'help' if user_lang != 'zh' else '帮助'),
-                        ('create   [bold magenta]<name>', 'create a Qpro project' if user_lang != 'zh' else '创建Qpro项目'),
-                        ('update', 'update Qpro' if user_lang != 'zh' else '更新Qpro'),
-                        ('adjust', 'adjust configure' if user_lang != 'zh' else '调整配置表'),
-                        ('ssh', 'login server by ssh' if user_lang != 'zh' else '通过SSH登录远程映射'),
-                        (
-                            'scp [bold magenta]<path>',
-                            'upload path to default server target' if user_lang != 'zh' else '上传路径到默认的远程映射对应位置'
-                        ),
-                        (
-                            'smv [bold magenta]<path>',
-                            'delete after scp' if user_lang != 'zh' else '上传完成后删除文件或目录'
-                        ),
-                        (
-                            'scp-init',
-                            'upload all of project to server target' if user_lang != 'zh' else '上传当前全部内容到远程映射'
-                        ),
-                        (
-                            'get [bold magenta]<path>',
-                            'download file from server target' if user_lang != 'zh' else '从远程映射下载'
-                        ),
-                        (
-                            'del [bold magenta]<path>',
-                            'delete path in project' if user_lang != 'zh' else '同时删除本地及远程映射文件或目录'
-                        ),
-                        ('del-all', 'delete Qpro project' if user_lang != 'zh' else '销毁当前Qpro项目(本地+远程)'),
-                        ('ls  [bold magenta]<path>', 'list element in path' if user_lang != 'zh' else '展示路径中的子项'),
-                        ('enable-complete', 'enable complete' if user_lang != 'zh' else '启用Commander类的自动补全'),
-                        ('tmpm *', 'manage your template' if user_lang != 'zh' else '模板管理器'),
-                        ('qrun *', 'run your Qpro project' if user_lang != 'zh' else '运行器'),
-                        (
-                            'detector -<p/f><p/f>',
-                            'run beat detector for two source files' if user_lang != 'zh' else '对拍器'
-                        )],
+                         ('init', 'let current dir be a Qpro project!' if user_lang != 'zh' else '使当前目录成为Qpro项目'),
+                         ('-h', 'help' if user_lang != 'zh' else '帮助'),
+                         ('create [bold magenta]<name>', 'create a Qpro project' if user_lang != 'zh' else '创建Qpro项目'),
+                         ('update', 'update Qpro' if user_lang != 'zh' else '更新Qpro'),
+                         ('adjust', 'adjust configure' if user_lang != 'zh' else '调整配置表'),
+                         ('ssh', 'login server by ssh' if user_lang != 'zh' else '通过SSH登录远程映射'),
+                         (
+                             'scp [bold magenta]<path>',
+                             'upload path to default server target' if user_lang != 'zh' else '上传路径到默认的远程映射对应位置'
+                         ),
+                         (
+                             'smv [bold magenta]<path>',
+                             'delete after scp' if user_lang != 'zh' else '上传完成后删除文件或目录'
+                         ),
+                         (
+                             'scp-init',
+                             'upload all of project to server target' if user_lang != 'zh' else '上传当前全部内容到远程映射'
+                         ),
+                         (
+                             'get [bold magenta]<path>',
+                             'download file from server target' if user_lang != 'zh' else '从远程映射下载'
+                         ),
+                         (
+                             'del [bold magenta]<path>',
+                             'delete path in project' if user_lang != 'zh' else '同时删除本地及远程映射文件或目录'
+                         ),
+                         ('del-all', 'delete Qpro project' if user_lang != 'zh' else '销毁当前Qpro项目(本地+远程)'),
+                         ('ls  [bold magenta]<path>', 'list element in path' if user_lang != 'zh' else '展示路径中的子项'),
+                         ('enable-complete', 'enable complete' if user_lang != 'zh' else '启用Commander类的自动补全'),
+                         ('register-global', 'register global command' if user_lang != 'zh' else '注册全局命令'),
+                         ('gen-fig-script', 'generate fig autocomplete scripts' if user_lang != 'zh' else '生成Fig自动补全脚本'),
+                         ('gen-zsh-comp', 'generate zsh autocomplete scripts' if user_lang != 'zh' else '生成zsh自动补全脚本'),
+                         ('tmpm *', 'manage your template' if user_lang != 'zh' else '模板管理器'),
+                         ('qrun *', 'run your Qpro project' if user_lang != 'zh' else '运行器'),
+                         (
+                             'detector -<p/f><p/f>',
+                             'run beat detector for two source files' if user_lang != 'zh' else '对拍器'
+                         )],
                      'prefix': 'Qpro'})
     elif 'update' == sys.argv[1]:
         os.system('pip3 install Qpro --upgrade')
     elif sys.argv[1] in func:
-        func[sys.argv[1]]()
+        try:
+            func[sys.argv[1]]()
+        except SystemExit:
+            return
+        except Exception:
+            QproDefaultConsole.print_exception()
     elif sys.argv[1] == 'scp-init':
         scp_init(get_config()['server_target'])
     elif 'init' != sys.argv[1]:
